/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.portal.search.test.util.facet;

import com.liferay.portal.kernel.search.Field;
import com.liferay.portal.kernel.search.SearchContext;
import com.liferay.portal.kernel.search.facet.Facet;
import com.liferay.portal.kernel.search.facet.ModifiedFacet;
import com.liferay.portal.search.test.util.indexing.QueryContributors;

import java.util.Arrays;
import java.util.List;

import org.junit.Test;

/**
 * @author Bryan Engler
 */
public abstract class BaseModifiedFacetTestCase extends BaseFacetTestCase {

	@Test
	public void testRange() throws Exception {
		addDocument("20170102000000");
		addDocument("20170104000000");
		addDocument("20170106000000");

		assertFacet(
			"[20170101000000 TO 20170105000000]",
			Arrays.asList("[20170101000000 TO 20170105000000]=2"));
	}

	protected void assertFacet(String customRange, List<String> expectedTerms)
		throws Exception {

		assertFacet(
			searchContext -> createFacet(searchContext, customRange),
			QueryContributors.dummy(), expectedTerms);
	}

	protected Facet createFacet(
		SearchContext searchContext, String customRange) {

		Facet facet = initFacet(new ModifiedFacet(searchContext));

		searchContext.setAttribute(facet.getFieldId(), customRange);

		return facet;
	}

	@Override
	protected String getField() {
		return Field.MODIFIED_DATE;
	}

}