/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.portal.search.test.util.filter;

import com.liferay.portal.kernel.search.Field;
import com.liferay.portal.kernel.search.Hits;
import com.liferay.portal.kernel.search.Query;
import com.liferay.portal.kernel.search.SearchContext;
import com.liferay.portal.kernel.search.filter.BooleanFilter;
import com.liferay.portal.kernel.search.filter.Filter;
import com.liferay.portal.kernel.search.filter.TermsFilter;
import com.liferay.portal.search.test.util.DocumentsAssert;
import com.liferay.portal.search.test.util.IdempotentRetryAssert;
import com.liferay.portal.search.test.util.indexing.BaseIndexingTestCase;
import com.liferay.portal.search.test.util.indexing.DocumentCreationHelpers;

import java.util.Arrays;
import java.util.List;
import java.util.concurrent.TimeUnit;

import org.junit.Test;

/**
 * @author André de Oliveira
 */
public abstract class BaseTermsFilterTestCase extends BaseIndexingTestCase {

	@Test
	public void testKeywordField() throws Exception {
		String fieldName = Field.FOLDER_ID;

		addDocuments(
			value -> DocumentCreationHelpers.singleKeyword(fieldName, value),
			Arrays.asList("One", "Two", "Three"));

		TermsFilter termsFilter = new TermsFilter(fieldName);

		termsFilter.addValues("Two", "Three");

		assertSearch(termsFilter, fieldName, Arrays.asList("Two", "Three"));
	}

	protected void assertSearch(
			Filter filter, String fieldName, List<String> expectedValues)
		throws Exception {

		IdempotentRetryAssert.retryAssert(
			10, TimeUnit.SECONDS,
			() -> doAssertSearch(filter, fieldName, expectedValues));
	}

	protected Void doAssertSearch(
			Filter filter, String fieldName, List<String> expectedValues)
		throws Exception {

		SearchContext searchContext = createSearchContext();

		Query query = getDefaultQuery();

		BooleanFilter booleanFilter = new BooleanFilter();

		booleanFilter.add(filter);

		query.setPreBooleanFilter(booleanFilter);

		Hits hits = search(searchContext, query);

		DocumentsAssert.assertValues(
			(String)searchContext.getAttribute("queryString"), hits.getDocs(),
			fieldName, expectedValues);

		return null;
	}

}